import React, {useEffect, useState, useContext} from "react";

import SlidesList      from "./SlidesList";
import Mode            from "../models/Mode";
import Slide           from "./Slide";
import KeyboardControl from "./KeyboardControl";
import Blackout        from "./Blackout";
import NoFile          from "./NoFile";
import Toolbar         from "./Toolbar";
import Tips            from "./Tips";

import SlideContext    from "../shared/SlideContext";
import SettingsContext from "../shared/SettingsContext";

const Ation = () => {
	const {font, highlightColor}  = useContext(SettingsContext);
	const [mode, setMode]         = useState(Mode.NORMAL);
	const [deck, setDeck]         = useState([]);
	const [slide, setSlide]       = useState(0);
	const [basePath, setBasePath] = useState("");
	const [showTips, setShowTips] = useState(false);
	const [version, setVersion]   = useState("0.0.0");

	useEffect(() => {
		window.api.onFileOpen(presentation => {
			window.api.clearCache();
			const [newBasePath, slideDeck] = presentation;
			if (!slideDeck)
				return;
			if (slide >= slideDeck.length)
				setSlide(0);
			setBasePath(newBasePath);
			setDeck(slideDeck);
		});
		window.api.onFileClose(() => {
			setBasePath("");
			setSlide(0);
			setDeck([]);
		});
		(async set => set(await window.api.appVersion()))(setVersion);
		//(async setFont => setFont(await window.appSettings.get("font", "Iosevka")))(setFont);

	}, [basePath, slide]);

	const openFile = () => {
		window.api.openFileDialog();
	}

	return (
		<>
			{deck.length < 1 ?
				<NoFile openFile={openFile} />
			: (
				<SlideContext.Provider value={{slide, setSlide, mode, setMode, basePath, slideCount : deck.length}}>
					<section className={`window${mode === Mode.PRESENT ? " fullscreen" : ""}`}>
						<Toolbar openFile={openFile} setShowTips={setShowTips} version={version} />
						<SlidesList deck={deck} />
						<main className="main" style={{"--color-hightlight" : highlightColor}}>
							<Slide data={deck[slide] || null} style={{fontFamily : font}}/>
						</main>
						<Tips show={showTips} />
					</section>
					<Blackout show={mode === Mode.BLACKOUT} />
					<KeyboardControl mode={mode} setMode={setMode} deck={deck} openFile={openFile} setShowTips={setShowTips} />
				</SlideContext.Provider>
			)}
		</>
	);
};

export default Ation;